#
# Generates alignments with bwa
#

# Read 1.
R1 ?= reads/read1.fq

# Read 2.
R2 ?= reads/read2.fq

# The reference genome.
REF ?= refs/genome.fa

# Name of the BAM file.
BAM ?= bam/alignment.bam

# Name of the BAM index file.
BAI ?= ${BAM}.bai

# The BAM statistics file.
BAM_STAT ?= $(basename ${BAM}).stats.txt

# The indexed reference genome.
IDX = idx/$(notdir ${REF})

# File in the index directory.
IDX_FX = ${IDX}.ann

# Alignment mode.
MODE ?= PE

# Number of CPUS
NCPU ?= 2

# Additional flags to pass to BWA.
BWA_FLAGS ?= -t ${NCPU}

# Sam filter flags to filter the BAM file before sorting.
SAM_FLAGS ?=

# Set the values for the read groups.
ID ?= run
SM ?= sample
LB ?= library
PL ?= ILLUMINA
RG ?= '@RG\tID:${ID}\tSM:${SM}\tLB:${LB}\tPL:${PL}'

# Targets that are not files.
.PHONY: align align! install usage index

# The first target is always the help.
usage::
	@echo ""
	@echo "# src/bwa.mk: aligns reads using BWA"
	@echo ""
	@echo "  make align"
	@echo ""
	@echo "  in:  MODE=${MODE}"
	@echo "  in:  REF=${REF}"
	@echo "  in:  IDX=${IDX}"
	@echo "  in:  R1=${R1}"
ifeq ($(MODE), PE)
	@echo "  in:  R2=${R2}"
endif
	@echo ""
	@echo "  out: BAM=${BAM}"
	@echo ""
	@echo "  cmd: ${CMD_BWA}"
	@echo ""

# Index the reference genome.
${IDX_FX}:
	@mkdir -p $(dir $@)
	bwa index -p ${IDX} ${REF}

# Generate the index.
index: ${IDX_FX}
	@echo "# Index: ${IDX}"

# Remove the index.
index!: ${IDX_FX}
	rm -rf ${IDX_FX}

# Different commands depending on library type.
ifeq ($(MODE), PE)
CMD_BWA ?= bwa mem ${BWA_FLAGS} -R ${RG} ${IDX} ${R1} ${R2} | samtools view -b ${SAM_FLAGS} | samtools sort -@ ${NCPU} > ${BAM}
else
CMD_BWA ?= bwa mem ${BWA_FLAGS} -R ${RG}  ${IDX} ${R1} | samtools view -b ${SAM_FLAGS} | samtools sort -@ ${NCPU} > ${BAM}
endif

# Different dependenices depending on library type.
ifeq ($(MODE), PE)
${BAM}: ${IDX_FX} ${R1} ${R2}
	@mkdir -p $(dir $@)
	${CMD_BWA}
else
${BAM}: ${IDX_FX} ${R1}
	@mkdir -p $(dir $@)
	${CMD_BWA}
endif
	samtools index ${BAM}

# Create the BAM index file.
${BAI}: ${BAM}
	samtools index ${BAM}

${BAM_STAT}: ${BAI}
	samtools flagstat ${BAM} > ${BAM_STAT}

# Display the BAM statistics.
align: ${BAM_STAT}
	@echo ""
	@cat ${BAM_STAT}
	@echo ""
	@echo "# BAM=${BAM}"
	@echo ""

# Remove the BAM file.
align!:
	@rm -rf ${BAM} ${BAI} ${BAM_STAT}

# Install required software.
install::
	@echo mamba install bwa samtools seqkit
