#
# Trims FASTQ files and runs FASTQC on them.
#
# Parameters:
#
# 	S1, S2 = original (source) read pairs
# 	R1, R2 = trimmed read pair names

# Read layout: single-end (SE) or paired-end (PE)
MODE ?= PE

# The first read always exists.
S1 ?= reads/read_1.fq
R1 = $(basename ${S1})_trim.fq

# Paired end mode.
ifeq ($(MODE), PE)
S2 ?= reads/read_2.fq
R2 = $(basename ${S2})_trim.fq
endif

# Number of CPUS to use.
NCPU ?= 2

# Adapter file name.
ADAPTER_FILE ?= $(basename ${R1}).adapter.fa

# The adapter sequence for trimming.
SEQ ?= AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC

# The statistics on the reads files.
READ_STATS ?= $(basename ${R1})_stats.log

# Targets that are not valid files.
.PHONY: all clean trim install help

# Print usage
usage::
	@echo ""
	@echo "# src/fastp.mk: trim FASTQ reads"
	@echo ""
	@echo "  make trim"
	@echo ""
	@echo "  in:  MODE=${MODE}"
	@echo "  in:  S1=${S1}"
ifeq ($(MODE), PE)
	@echo "  in:  S2=${S2}"
endif
	@echo ""
	@echo "  out: R1=${R1}"
ifeq ($(MODE), PE)
	@echo "  out: R2=${R2}"
endif
	@echo ""
	@echo "  cmd: ${CMD_FASTP}"
	@echo ""

# Making adapter explicitly.
${ADAPTER_FILE}:
	mkdir -p  ${FASTQ_DIR}
	@echo "# Creating ADAPTER_FILE=${ADAPTER_FILE}"
	@echo ">sequence" > ${ADAPTER_FILE}
	@echo ${SEQ} >> ${ADAPTER_FILE}

# Create the adapter
adapter: ${ADAPTER_FILE}
	@cat ${ADAPTER_FILE}
	@echo "# ADAPTER_FILE=${ADAPTER_FILE}"

# Remove the adapter file.
adapter!:
	@rm -f ${ADAPTER_FILE}

# Default trimming options.
FASTP_FLAGS ?= -q 15 -u 40 -l 25 -l 25 --trim_poly_g --trim_poly_x

# FASTP html report.
FASTP_HTML = $(basename ${R1}).html

# FASTP trimming log.
TRIM_LOG = $(basename ${R1})_fastp.log

# FASTP json data.
FASTP_JSON = $(basename ${R1}).html

# How to pass custom adapter sequences to FASTP
# --adapter_fasta ${ADAPTER_FILE}

# Other trimming options, more like Trimmomatic
#FASTP_FLAGS = --cut_right --cut_right_window_size 4 --cut_right_mean_quality 30 --length_required 50

# Different rules for single-end and paired-end reads.
ifeq ($(MODE), PE)
CMD_FASTP ?= fastp -i ${S1} -I ${S2} -o ${R1} -O ${R2}  -w ${NCPU} -j /dev/null -h ${FASTP_HTML} \
		${FASTP_FLAGS}
else
CMD_FASTP ?= fastp -i ${S1} -o ${R1}  -w ${NCPU} -j /dev/null -h ${FASTP_HTML} \
		${FASTP_FLAGS}
endif

# Different rules for single-end and paired-end reads.
ifeq ($(MODE), PE)
${R1} ${R2}: ${S1} ${S2}
	${CMD_FASTP} 2> ${TRIM_LOG}

${TRIM_LOG}: ${R1} ${R2}

${READ_STATS}:: ${R1} ${R2}
	@seqkit stats ${R1} ${R2} > ${READ_STATS}
else
${R1}: ${S1}
	${CMD_FASTP} 2> ${TRIM_LOG}

${TRIM_LOG}: ${R1}

${READ_STATS}:: ${R1}
	@seqkit stats ${R1}  > ${READ_STATS}
endif

# Runs the trimmer.
trim: ${READ_STATS} ${TRIM_LOG}
	@echo ""
	@cat ${TRIM_LOG}
	@echo ""
	@cat ${READ_STATS}
	@echo ""
	@echo "# Trim log: ${TRIM_LOG}"
	@echo "# Read log: ${READ_STATS}"
	@echo "# HTML log: ${FASTP_HTML}"

# Removes the trimmed files.
trim!:
	@rm -f ${R1} ${R2} ${READ_STATS} ${TRIM_LOG}


# Installation instuctions
install::
	@echo mamba
