#
# Generates alignments with hisat2
#
# Sequencing read 1.
R1 ?= reads/read1.fq

# Sequencing read 2.
R2 ?= reads/read2.fq

# The reference genome.
REF ?= refs/genome.fa

# Name of the BAM file.
BAM ?= bam/alignment.bam

# The bam index file.
BAI = ${BAM}.bai

# The BAM statistics file.
BAM_STAT ?= ${BAM}.stats.txt

# The indexed reference genome.
IDX = idx/$(notdir ${REF})

# File in the index directory.
IDX_FX = ${IDX}.1.ht2

# Alignment mode.
MODE ?= PE

# Number of CPUS
NCPU ?= 2

# Additional hisat2 options.
HS2_FLAGS = --sensitive

# Sam flags to filter the BAM file before sorting.
SAM_FLAGS ?=

# Set the values for the read groups.
ID ?= run
SM ?= sample
LB ?= library
PL ?= ILLUMINA
RG ?= "@RG\tID:{$ID}\tSM:${SM}\tLB:${LB}\tPL:${PL}"

# Targets that are not files.
.PHONY: align align! install usage index

# The first target is always the help.
usage::
	@echo ""
	@echo "# src/hisat2.mk: aligns read using hisat2 "
	@echo ""
	@echo "  make align"
	@echo ""
	@echo "  in:  MODE=${MODE}"
	@echo "  in:  REF=${REF}"
	@echo "  in:  IDX=${IDX}"
	@echo "  in:  R1=${R1}"
ifeq ($(MODE), PE)
	@echo "  in:  R2=${R2}"
endif
	@echo ""
	@echo "  out: BAM=${BAM}"
	@echo ""
	@echo "  cmd: ${HS2_CMD}"
	@echo ""

# Index the reference.
${IDX_FX}:
	mkdir -p $(dir $@)
	hisat2-build ${REF} ${IDX}

# Different modes depending on single end or paired end reads.
ifeq ($(MODE), PE)
HS2_CMD ?= hisat2 ${HS2_FLAGS} -x ${IDX} -1 ${R1} -2 ${R2} | samtools view -h ${SAM_FLAGS} | samtools sort -@ ${NCPU}  > ${BAM}
else
HS2_CMD ?= hisat2 ${HS2_FLAGS} -x ${IDX} -U ${R1}  | samtools view -h ${SAM_FLAGS} | samtools sort -@ ${NCPU} > ${BAM}
endif

ifeq ($(MODE), PE)
${BAM}: ${IDX_FX} ${R1} ${R2}
	mkdir -p $(dir $@)
	${HS2_CMD}
	samtools index ${BAM}
else
${BAM}: ${IDX_FX} ${R1}
	mkdir -p $(dir $@)
	${HS2_CMD}
	samtools index ${BAM}
endif

${BAM_STAT}: ${BAM}
	samtools flagstat ${BAM} > ${BAM_STAT}

# Generate the index.
index: ${IDX_FX}
	@echo "# Index: ${IDX}"

# Remove the index.
index!: ${IDX_FX}
	rm -rf ${IDX_FX}

# Target to trigger the alignment.
align: ${BAM_STAT}
	@echo ""
	@cat ${BAM_STAT}
	@echo ""
	@echo "# BAM=${BAM}"
	@echo ""

# Remove the BAM file.
align!:
	@rm -rf ${BAM} ${BAI} ${BAM_STAT}

# Install required software.
install::
	@echo mamba install bowtie2 samtools seqkit

