#
# Generates alignments with minimap2
#

# Sequencing read 1.
R1 ?= reads/read1.fq

# Sequencing read 2.
R2 ?= reads/read2.fq

# The reference genome.
REF ?= refs/genome.fa

# Name of the BAM file.
BAM ?= bam/alignment.bam

# The bam index file.
BAI = ${BAM}.bai

# The BAM statistics file.
BAM_STAT ?= ${BAM}.stats.txt

# The indexed reference genome.
IDX = idx/$(notdir ${REF}).mmi

# Alignment mode.
MODE ?= PE

# Number of CPUS
NCPU ?= 2

# Additional minimap2 options.
MP2_FLAGS = --eqx -x sr -a -MD -c --cs -t ${NCPU} --secondary=no --sam-hit-only

# Sam flags to filter the BAM file before sorting.
SAM_FLAGS ?=

# Set the values for the read groups.
ID ?= run
SM ?= sample
LB ?= library
PL ?= ILLUMINA
RG ?= "@RG\\tID:${ID}\tSM:${SM}\tLB:${LB}\tPL:${PL}"

# Targets that are not files.
.PHONY: align align! install usage

# The first target is always the help.
usage::
	@echo ""
	@echo "# src/minimap2.mk: aligns read using minimap2 "
	@echo ""
	@echo "  make align"
	@echo ""
	@echo "  in:  MODE=${MODE}"
	@echo "  in:  REF=${REF}"
	@echo "  in:  IDX=${IDX}"
	@echo "  in:  R1=${R1}"
ifeq ($(MODE), PE)
	@echo "  in:  R2=${R2}"
endif
	@echo ""
	@echo "  out: BAM=${BAM}"
	@echo ""
	@echo "  cmd: ${MP2_CMD}"
	@echo ""
# Index the reference genome.
${IDX}:
	mkdir -p $(dir $@)
	minimap2 -t ${NCPU} -x sr -d ${IDX} ${REF}

# Generate the index.
index: ${IDX}
	@echo "# Index: ${IDX}"

# Remove the index.
index!: ${IDX}
	rm -rf ${IDX}

# Different modes depending on single end or paired end reads.
ifeq ($(MODE), PE)
MP2_CMD ?= minimap2 ${MP2_FLAGS} -R ${RG} ${IDX} ${R1} ${R2} | samtools view -h ${SAM_FLAGS} | samtools sort -@ ${NCPU} > ${BAM}
else
MP2_CMD ?= minimap2 ${MP2_FLAGS} -R ${RG} ${IDX} ${R1}  | samtools view -h ${SAM_FLAGS} | samtools sort -@ ${NCPU} > ${BAM}
endif

# Requirements also depend on the mode.
ifeq ($(MODE), PE)
${BAM}: ${IDX} ${R1} ${R2}
	@mkdir -p $(dir $@)
	${MP2_CMD}
	samtools index ${BAM}
else
${BAM}: ${IDX} ${R1}
	@mkdir -p $(dir $@)
	${MP2_CMD}
	samtools index ${BAM}
endif

# Generate the index file.
${BAI}: ${BAM}
	samtools index ${BAM}

# Generate the BAM statistics file.
${BAM_STAT}: ${BAI}
	@mkdir -p $(dir $@)
	samtools flagstat ${BAM} > ${BAM_STAT}

# Target to trigger the alignment.
align: ${BAM_STAT}
	@echo ""
	@cat ${BAM_STAT}
	@echo ""
	@echo "# BAM=${BAM}"
	@echo ""

# Remove the BAM file.
align!:
	@rm -rf ${BAM} ${BAI} ${BAM_STAT}

# Install required software.
install::
	@echo mamba install minimap2 samtools
