#
# Downloads sequencing reads from SRA.
#

# The directory that stores FASTQ reads that we operate on.
FASTQ_DIR ?= reads

# SRR number (sequencing run from the Ebola outbreak data of 2014)
SRR ?= SRR1553591

# Location of the adapter sequence.
ADAPTER_FILE ?= $(FASTQ_DIR)/${SRR}_adapter.fa

# Read layout: single-end (SE) or paired-end (PE)
MODE ?= PE

# How many reads to download (N=ALL downloads everything).
N ?= 1000

# The statistics for the source reads.
SOURCE_STATS ?= $(basename ${S1})_${MODE}_stats.txt

# The default name of the alignment file for SRA data
BAM ?= bam/${SRR}.bam

# Name of the VCF with all variants.
VCF ?= vcf/${SRR}.vcf.gz

# Targets that are not files.
.PHONY: usage install fastq fastq!

usage::
	@echo ""
	@echo "# sra.mk: downloads FASTQ reads from SRA"
	@echo ""
	@echo "  make fastq"
	@echo ""
	@echo "  in:  MODE=${MODE}"
	@echo "  in:  SRR=${SRR}"
	@echo "  in:  N=${N} (N=ALL for full download)"
	@echo ""
	@echo "  out: S1=${S1}"
ifeq ($(MODE), PE)
	@echo "  out: S2=${S2}"
endif
	@echo ""
	@echo "  cmd: ${CMD_SRA}"
	@echo ""

# Different SPLIT flags for different library layouts single-end/paired end.
ifeq ($(MODE), PE)
SPLIT_MODE ?= --split-3
S1 ?= ${FASTQ_DIR}/${SRR}_1.fq
S2 ?= ${FASTQ_DIR}/${SRR}_2.fq
# Setting R2 just to interface with other tools later on.
R2 ?= ${S2}
else
SPLIT_MODE ?= --split-spot
S1 ?= ${FASTQ_DIR}/${SRR}.fq
endif

# Setting R1 just to interface with other tools later on.
R1 ?= ${S1}

# Set the command.
ifeq ($(N), ALL)
CMD_SRA ?= fasterq-dump -p -f ${SPLIT_MODE} ${SRR} -O ${FASTQ_DIR}
#CMD_SRA ?= fastq-dump -F ${SPLIT_MODE} ${SRR} -O ${FASTQ_DIR}
else
CMD_SRA ?= fastq-dump -F -X ${N} ${SPLIT_MODE} ${SRR} -O ${FASTQ_DIR}
endif

# Different tools used whether we need all reads or a subset.
ifeq ($(MODE), PE)
${S1} ${S2}:
	@mkdir -p ${FASTQ_DIR}
	${CMD_SRA}
	@mv -f ${FASTQ_DIR}/${SRR}_1.fastq ${S1}
	@mv -f ${FASTQ_DIR}/${SRR}_2.fastq ${S2}
else
${S1}:
	@mkdir -p ${FASTQ_DIR}
	${CMD_SRA}
	@mv -f ${FASTQ_DIR}/${SRR}.fastq ${S1}
endif

# Generate the source read statistics.
${SOURCE_STATS}:: ${S1} ${S2}
	@seqkit stats  ${S1} ${S2} > ${SOURCE_STATS}

# Target to download all the data.
fastq:: ${SOURCE_STATS}
	@echo ""
	@cat ${SOURCE_STATS}
	@echo ""

# Removes the SRA files.
fastq!::
	@rm -f ${S1} ${S2} ${SOURCE_STATS}

install::
	@echo mamba install sra-tools seqkit


