
# The FASTA file for the genome
GENOME_URL ?= http://ftp.ensembl.org/pub/release-104/fasta/homo_sapiens/dna/Homo_sapiens.GRCh38.dna.chromosome.22.fa.gz

# The FASTA file for the CDS (transcripts)
CDS_URL ?= http://ftp.ensembl.org/pub/release-104/fasta/homo_sapiens/cds/Homo_sapiens.GRCh38.cds.all.fa.gz

# The GTF file with annotations.
GTF_URL ?= http://ftp.ensembl.org/pub/release-104/gtf/homo_sapiens/Homo_sapiens.GRCh38.104.gtf.gz

# The reads obtained from the biostar handbook website.
READS_URL ?= http://data.biostarhandbook.com/rnaseq/projects/griffith/griffith-data.tar.gz

# The reference genome.
REF ?= refs/Homo_sapiens.GRCh38.dna.chromosome.22.fa

# Annotations.
GTF ?= refs/Homo_sapiens.GRCh38.104.gtf

# Transcript sequences.
CDS ?= refs/Homo_sapiens.GRCh38.cds.all.fa

# Targets that are not files.
.PHONY: all download align coverage salmon

# Prints the usage.
usage:
	@echo ""
	@echo "# Consult the makefile for usage"
	@echo ""

# Download each reference file.
download:
	@make -s -f src/download.mk URL=${GENOME_URL} get
	@make -s -f src/download.mk URL=${CDS_URL} get
	@make -s -f src/download.mk URL=${GTF_URL} get
	@make -s -f src/download.mk URL=${READS_URL} get

# Unpack the downloaded data
unpack:
	mkdir -p results
	tar zxvf refs/griffith-data.tar.gz
	gunzip -c ${REF}.gz > ${REF}
	gunzip -c ${GTF}.gz > ${GTF}
	gunzip -c ${CDS}.gz > ${CDS}

# Create roots over from which filenames are generated.
ids.txt:
	echo HBR_1 HBR_2 HBR_3 UHR_1 UHR_2 UHR_3 | tr ' ' '\n' > ids.txt

# Generate indices for the reference genomes.
index:
	make -s -f src/hisat2.mk REF=${REF} index
	make -s -f src/salmon.mk CDS=${CDS} index

# Run the alignments on each sample.
align: ids.txt
	cat ids.txt | parallel make -f src/hisat2.mk REF=${REF} R1=reads/{}_R1.fq R2=reads/{}_R2.fq BAM=bam/{}.bam align

# Turn each BAM file into bedGraph coverage.
coverage: ids.txt
	samtools faidx ${REF}
	cat ids.txt | parallel -v "bedtools genomecov -ibam  bam/{}.bam -split -bg  > bam/{}.bg"
	cat ids.txt | parallel -v "bedGraphToBigWig bam/{}.bg  ${REF}.fai bam/{}.bw"

# Run the feature counting.
count:
	featureCounts -g gene_id -p -a ${GTF} -o results/hisat2.txt bam/HBR_?.bam bam/UHR_?.bam

	@echo  ""
	@echo "# featureCounts output: results/hisat2.txt"
	@echo ""

# Run the quantification.
salmon: ids.txt

	cat ids.txt | parallel -u make -f src/salmon.mk CDS=${CDS} R1=reads/{}_R1.fq R2=reads/{}_R2.fq BAM=bam/{}.bam PREFIX=salmon/{} align

	bio combine salmon/HBR_? salmon/UHR_? > results/salmon.txt

	@echo  ""
	@echo "# salmon output: results/salmon.txt"
	@echo ""


# Run the differential expression analysis.
edger:
	cat results/hisat2.txt | edger.r 3x3 > results/hisat2-edger.csv
	cat results/hisat2-edger.csv | heatmap.r > results/hisat2-edger.pdf

	cat results/salmon.txt | edger.r 3x3 > results/salmon-edger.csv
	cat results/salmon-edger.csv | heatmap.r > results/salmon-edger.pdf

# One stop run of all tasks.
all: align coverage count salmon edger

# Required tooling
install:
	echo mamba install hisat2 samtools bedtools bedGraphToBigWig featureCounts salmon
