#
# Downloads NCBI data vie Entrez API
#


# Accession number at NCBI (default Ebola Mayinga Zaire 1977)
ACC ?= AF086833

# The Genbank file.
GBK ?= refs/${ACC}.gb

# The reference genome.
REF ?= refs/${ACC}.fa

# The GFF annotation file.
GFF ?= refs/${ACC}.gff

# A statistics file on the reference genome.
REF_STATS ?= ${REF}.stats.txt

# Project number (Zaire ebolavirus sample sequencing from the 2014 outbreak in Sierra Leone, West Africa.)
PRJN ?= PRJNA257197

# Project number runinfo.
PRJN_CSV ?= ${PRJN}_runinfo.csv

# Project number SRR numbers.
PRJN_IDS ?= ${PRJN}_srr.txt

# Targets that are not files.
.PHONY: genome runinfo help

# General usage information.
usage::
	@echo ""
	@echo "# ncbi.mk: download FASTA and GFF data from NCBI"
	@echo ""
	@echo "  make genome"
	@echo ""
	@echo "  in:  ACC=${ACC}"
	@echo ""
	@echo "  out: REF=${REF}"
	@echo "  out: GFF=${GFF}"
	@echo ""
	@echo "  cmd: bio fetch ${ACC} > ${GBK}"
	@echo ""
# Download the Genbank file.
${GBK}:
	@mkdir -p $(dir $@)
	bio fetch ${ACC} > $@

# Create the reference file. Also creates the GFF file.
${REF}: ${GBK} ${GFF}
	@mkdir -p $(dir $@)
	bio fasta ${GBK} > ${REF}

# Download the GFF file.
${GFF}: ${GBK}
	@mkdir -p $(dir $@)
	bio gff ${GBK} > ${GFF}

# Generates the genome statistics.
${REF_STATS}:: ${REF}
	@mkdir -p $(dir $@)
	@seqkit stats ${REF} > ${REF_STATS}

# Target to download all the data.
genome:: ${REF} ${GFF} ${GBK} ${REF_STATS}
	@echo ""
	@cat ${REF_STATS}
	@echo ""
	@echo "# GBK=${GBK} (`cat ${GBK} | wc -l | tr -d ' '` lines)"
	@echo "# REF=${REF} (`cat ${REF} | wc -l | tr -d ' '` lines)"
	@echo "# GFF=${GFF} (`cat ${GFF} | wc -l | tr -d ' '` lines)"
	@echo ""

# Remove genome files.
genome!::
	@rm -rf ${GBK} ${REF} ${GFF} ${REF_STATS}

# Project run information.
${PRJN_CSV}:
	esearch -db sra -query ${PRJN} | efetch -format runinfo > ${PRJN_CSV}

# Project SRR ids.
${PRJN_IDS}: ${PRJN_CSV}
	cat ${PRJN_CSV} | csvtk cut -f Run | grep -v Run > ${PRJN_IDS}

# Download run information.
runinfo:: ${PRJN_IDS}
	@cat ${PRJN_CSV} | head -3

# Remove run information.
runinfo!::
	@rm -rf ${PRJN_CSV} ${PRJN_IDS}

# Installation instructions
install::
	@echo pip install bio --upgrade
	@echo "mamba install csvtk seqkit entrez-direct"
